"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _chalk = _interopRequireDefault(require("chalk"));

var _path = _interopRequireDefault(require("path"));

var _replace = _interopRequireDefault(require("./replace"));

var _minimatch = _interopRequireDefault(require("minimatch"));

var _globParent = _interopRequireDefault(require("glob-parent"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class File {
  locate(filepath, options = {}) {
    if (_path.default.isAbsolute(filepath)) {
      return filepath;
    }

    options = Object.assign({
      cwd: process.cwd()
    }, options);
    return _path.default.join(options.cwd, filepath);
  }

  exists(filename, options) {
    try {
      return _fsExtra.default.statSync(this.locate(filename, options)).isFile();
    } catch (e) {
      if (e.code === 'ENOENT') {
        return false;
      }

      throw e;
    }
  }

  isDirectory(directory, options) {
    try {
      return _fsExtra.default.statSync(this.locate(directory, options)).isDirectory();
    } catch (e) {
      if (e.code === 'ENOENT') {
        return false;
      }

      throw e;
    }
  }

  remove(filepath, options) {
    return new Promise((resolve, reject) => {
      _fsExtra.default.remove(this.locate(filepath, options), err => {
        if (err) {
          reject(err);
        } else {
          resolve();
        }
      });
    });
  }

  read(filename, options = {}) {
    options = Object.assign({
      encoding: 'utf-8'
    }, options);
    return new Promise((resolve, reject) => {
      _fsExtra.default.readFile(this.locate(filename, options), (err, content) => {
        if (err) {
          reject(err);
        } else {
          resolve(content);
        }
      });
    });
  }

  copy(src, dest, options) {
    /*eslint no-unneeded-ternary: "off"*/
    options = Object.assign({
      clobber: options.override ? true : false,
      preserveTimestamps: false
    }, options);
    delete options.override;
    return new Promise((resolve, reject) => {
      _fsExtra.default.copy(this.locate(src, options), this.locate(dest, options), options, err => {
        if (err) {
          reject(err);
        } else {
          resolve();
        }
      });
    });
  }

  replace(src, dest, rules, options) {
    options = Object.assign({
      encoding: 'utf-8',
      override: true
    }, options);
    return this.read(src, options).then(content => {
      content = (0, _replace.default)(content.toString(options.encoding), rules);
      return this.write(dest, content, options);
    });
  }

  write(src, content, options) {
    options = Object.assign({
      encoding: 'utf-8',
      override: true
    }, options);
    let override = options.override;
    delete options.override;
    return new Promise((resolve, reject) => {
      let filepath = this.locate(src, options);

      _fsExtra.default.ensureFile(filepath, err => {
        if (err) {
          reject(err);
        }

        if (override) {
          this.remove(filepath, options).then(() => {
            _fsExtra.default.writeFile(filepath, content, options || {}, err => {
              if (err) {
                reject(err);
              } else {
                resolve();
              }
            });
          }).catch(reject);
        } else {
          _fsExtra.default.writeFile(filepath, content, options || {}, err => {
            if (err) {
              reject(err);
            } else {
              resolve();
            }
          });
        }
      });
    });
  }

  readJSON(filepath, defaults, options) {
    options = Object.assign({
      encoding: 'utf-8'
    }, options);
    filepath = this.locate(filepath, options);

    if (this.exists(filepath)) {
      try {
        return JSON.parse(_fsExtra.default.readFileSync(filepath));
      } catch (error) {
        throw new Error('Could not parse JSON in file: ' + filepath + '. Detail: ' + error.message);
      }
    } else {
      return defaults;
    }
  }

  cleanFiles(files, options) {
    options = Object.assign({
      verbose: false,
      log: console.log,
      ignoreError: true
    }, options);
    const log = options.verbose ? options.log : function () {};
    return new Promise((resolve, reject) => {
      if (!Array.isArray(files)) {
        files = [files];
      }

      let opts = {};

      if (options.cwd) {
        opts.cwd = options.cwd;
      }

      let all = files.map(target => {
        return this.remove(target, opts).then(log(_chalk.default.red('Removing: ') + target)).catch(err => {
          if (options.ignoreError === false) {
            reject(`Cannot clean '${target}': ${err.code}`);
          }
        });
      });
      Promise.all(all).then(resolve).catch(reject);
    });
  }

  copyFiles(files, options) {
    options = Object.assign({
      verbose: false,
      override: true,
      log: console.log,
      replaces: {},
      processes: {}
    }, options);
    const log = options.verbose ? options.log : function () {};
    return new Promise((resolve, reject) => {
      if (!Array.isArray(files)) {
        files = [files];
      }

      let opts = {
        override: options.override
      };

      if (options.cwd) {
        opts.cwd = options.cwd;
      }

      let all = files.map(target => {
        let promise = null;
        let replaces = {};

        if (options.replaces) {
          for (let glob in options.replaces) {
            if ((0, _minimatch.default)(target.dest, glob, {
              matchBase: true
            })) {
              Object.assign(replaces, options.replaces[glob]);
            }
          }
        }

        if (Object.keys(replaces).length === 0) {
          promise = this.copy(target.src, target.dest, opts);
        } else {
          promise = this.replace(target.src, target.dest, replaces, opts);
        }

        if (options.processes) {
          let processes = [];

          for (let glob in options.processes) {
            if ((0, _minimatch.default)(target.dest, glob, {
              matchBase: true
            })) {
              if (Array.isArray(options.processes[glob])) {
                processes = processes.concat(options.processes[glob]);
              } else {
                processes.push(options.processes[glob]);
              }
            }
          }

          if (processes.length !== 0) {
            promise = promise.then(() => {
              return this.read(target.dest, opts).then(content => {
                content = content.toString(options.encoding);
                processes.forEach(process => {
                  /* eslint no-eval: "off" */
                  if (typeof process === 'string') {
                    eval("process = " + process);
                  }

                  content = process(content, target.dest);
                });
                return this.write(target.dest, content, options);
              });
            });
          }
        }

        return promise.then(log(_chalk.default.green('Copying: ') + target.src + _chalk.default.cyan(' -> ') + target.dest)).catch(err => {
          reject(`Cannot copy ${target.src}'' to '${target.dest}': ${err.message}`);
        });
      });
      Promise.all(all).then(resolve).catch(reject);
    });
  }

  getFilenameWithIndex(filepath, index) {
    let parsed = _path.default.parse(filepath);

    return _path.default.join(parsed.root, parsed.dir, `${parsed.name}-${index}${parsed.ext}`);
  }

  getRelativeFromGlobs(filepath, globs) {
    let glob = this.matchGlob(filepath, globs);

    if (glob) {
      return _path.default.relative((0, _globParent.default)(glob), filepath);
    }

    return _path.default.basename(filepath);
  }

  matchGlob(filepath, globs) {
    if (!Array.isArray(globs)) {
      globs = [globs];
    }

    for (let i in globs) {
      if ((0, _minimatch.default)(filepath, globs[i])) {
        return globs[i];
      }
    }

    return false;
  }

}

const file = new File();
var _default = file;
exports.default = _default;
module.exports = exports.default;